==> gtkeditor-0.0.3.tar.gz: gtkeditor-0.0.3/gtkeditor.c <==
/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * The editor widget was written by Thomas Mailund, so bugs should be
 * reported to <mailund@daimi.au.dk>, not the gtk ppl.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>

#include <glib.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>

#include "gtkeditor.h"
#include "gtkeditor-internal.h"
#include "gtkeditor-syntaxtable.h"
#include "gtkeditor-regex.h"

#define TIME_DELAY    300	/* delay in milisec. before hilite */

/* widget stuff */
static void   gtk_editor_class_init          (GtkEditorClass   *klass);
static void   gtk_editor_init                (GtkEditor        *editor);
static void   gtk_editor_destroy             (GtkObject        *object);

/* text property data */
static gboolean cmp_prop_data                (MarkData         *d1,
					      MarkData         *d2);
static MarkData *clone_prop_data             (MarkData         *d);
static void free_prop_data                   (MarkData         *d);

/* search stuff */
static gint   search                         (GtkEditor        *editor,
					      guint             pos,
					      gchar            *string,
					      gboolean          casein);
static gint   search_back                    (GtkEditor        *editor,
					      guint             pos,
					      gchar            *string,
					      gboolean          casein);
/* hilit */
static gboolean hilite_when_idle             (GtkEditor        *editor);

/* indent */
static void   default_one_line_ind           (GtkEditor        *editor);
static void   indent_region                  (GtkEditor        *editor);


/* events */
static gint   gtk_editor_key_press           (GtkWidget        *widget,
					      GdkEventKey      *event);
static void   gtk_editor_changed_pre         (GtkEditor        *editor);
static void   gtk_editor_changed_post        (GtkEditor        *editor);


/* --<local data>--------------------------------------------------------- */
static GtkWidgetClass *parent_class = NULL;

/* --<widget initialization, constructor and destructor>------------------ */
guint
gtk_editor_get_type ()
{
  static guint editor_type = 0;

  if (!editor_type)
    {
      GtkTypeInfo editor_info =
      {
	"GtkEditor",
	sizeof (GtkEditor),
	sizeof (GtkEditorClass),
	(GtkClassInitFunc) gtk_editor_class_init,
	(GtkObjectInitFunc) gtk_editor_init,
	(GtkArgSetFunc) NULL,
        (GtkArgGetFunc) NULL,
      };

      editor_type = gtk_type_unique (gtk_text_get_type (), &editor_info);
    }

  return editor_type;
}

static void
gtk_editor_class_init (GtkEditorClass *class)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*) class;

  re_set_syntax (RE_SYNTAX_EMACS);

  /* init local data */
  parent_class = gtk_type_class (gtk_text_get_type ());

  /* setup signals */
  object_class->destroy = gtk_editor_destroy;
}

static void
gtk_editor_init (GtkEditor *editor)
{
  GtkText *text = GTK_TEXT (editor);

  gtk_text_set_property_data_functions (text,
					(gboolean(*)(gpointer,gpointer))cmp_prop_data,
					(gpointer(*)(gpointer))clone_prop_data,
					(void(*)(gpointer))free_prop_data);

  /* hilite */
  editor->stable = NULL;

  editor->hilite_when_idle = TRUE;
  editor->hilite_time = 0;
  editor->changed_from = editor->changed_to = 0;

  /* to keep track of changes, for hilite_when_idle */
  gtk_signal_connect (GTK_OBJECT (editor), "insert_text",
		      GTK_SIGNAL_FUNC (gtk_editor_changed_pre),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (editor), "delete_text",
		      GTK_SIGNAL_FUNC (gtk_editor_changed_pre),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (editor), "changed",
		      GTK_SIGNAL_FUNC (gtk_editor_changed_post),
		      NULL);

  /* indent */
  editor->one_line_ind = default_one_line_ind;
  gtk_signal_connect (GTK_OBJECT (editor), "key_press_event",
		      GTK_SIGNAL_FUNC (gtk_editor_key_press),
		      NULL);
}

GtkWidget*
gtk_editor_new (GtkAdjustment *hadj,
		GtkAdjustment *vadj)
{
  GtkEditor *editor;

  editor = gtk_type_new (gtk_editor_get_type ());

  gtk_text_set_adjustments (GTK_TEXT (editor), hadj, vadj);

  return GTK_WIDGET (editor);
}

static void
gtk_editor_destroy (GtkObject *object)
{
  g_return_if_fail (object != NULL);
  g_return_if_fail (GTK_IS_EDITOR (object));


  GTK_OBJECT_CLASS(parent_class)->destroy (object);
}

/* --<text property data>------------------------------------------------- */
static gboolean
cmp_prop_data (MarkData *d1, MarkData *d2)
{
  if (d1 == d2) return TRUE;

  if (d1 && d2)
    return ( (d1->nlvl == d2->nlvl) && (d1->ste == d2->ste) );
      
  return FALSE;
}

static MarkData *
clone_prop_data (MarkData *d)
{
  MarkData *new;
  if (!d) return NULL;
  new = g_new (MarkData, 1);
  new->nlvl = d->nlvl;
  new->ste = d->ste;
  return new;
}

static void
free_prop_data (MarkData *d)
{
  g_free (d);
}

MarkData*
_gtk_editor_new_mark_data (int nlvl, GtkEditorSTEntry *ste)
{
  MarkData *new = g_new (MarkData, 1);
  new->nlvl = nlvl;
  new->ste = ste;
  return new;
}

/* --<search'n'stuff>----------------------------------------------------- */
/* NB! There is really no reason why all this searching stuff couldn't
 * be moved to the text widget...the only reason I can think of is
 * that, the both gtktext and gtkeditor would have to deal with
 * regexps, but that doesn't seem too discouraging. We will have to
 * make them more 'public' though...the editor _needs_ some of the
 * static functions, so we can't hide them in the text widget. */

/* _gtk_editor_txteq -- compares 'text' at 'index' with 'string' */
gboolean
_gtk_editor_txteq (GtkText *text, char *string, int index)
{
  int diff;
  int len = strlen (string);

  if ( (index < text->gap_position) ) {
    /* before gap */

    /* does the string overlap the gap? */
    if ((index + strlen (string)) > text->gap_position) {
      diff = text->gap_position - index;
      return ( (strncmp (text->text+index, string, diff) == 0)
	       && (strncmp (text->text+text->gap_position+text->gap_size,
			    string+diff, len-diff) == 0) );
      
    } else {
      return ( strncmp (text->text+index, string, len) == 0 );
    }

  } else {
    /* after gap ... adjust index */

    index += text->gap_size;
    diff = text->text_end - index;
    return ( strncmp (text->text+index, string, MIN (diff,len)) == 0 );
  }
}

/* search -- searches in editor, from pos and forward, for string.  If
 * string is found, returns position, otherwise returns -1.
 * If casein is true, search is case insensitive. */
/* FIXME: This could and should be optimised. There is no need to
 * get_text all the time, is there?  We just have to be carefull with
 * the gap. ... use a modified version of _gtk_editor_txteq! */
static gint
search (GtkEditor *editor, guint pos, gchar *string, gboolean casein)
{
  gchar *buf = NULL;
  gint match = -1;
  gint text_len = gtk_text_get_length (GTK_TEXT (editor));
  gint len = strlen (string);

  g_return_val_if_fail (pos <= text_len, -1);
  if (pos == text_len) return -1; /* not illegal, but nothing to do */

  gtk_text_freeze (GTK_TEXT (editor));
  for (; pos < (text_len - len) + 1; pos++) {
    buf = gtk_editable_get_chars (GTK_EDITABLE (editor), pos, pos + len);
    if (!buf) return match;	/* somethings wrong */
    if (casein) {
      if (strcasecmp (buf, string) == 0) {
	match = pos;
	break;
      }
    } else {
      if (strcmp (buf, string) == 0) {
	match = pos;
	break;
      }
    }
    g_free (buf);
  }
  g_free (buf);
  gtk_text_thaw (GTK_TEXT (editor));

  return match;
}

/* gtk_editor_search_from_point -- searches for string, if found, goto
 * and select match. If casein is true, search is case insensitive. If
 * string is found it returns TRUE, otherwise FALSE. */
gboolean
gtk_editor_search_from_point (GtkEditor *editor, gchar *string, gboolean casein)
{
  gint new_pos;

  g_return_val_if_fail (editor != NULL, FALSE);
  g_return_val_if_fail (string != NULL, FALSE);

  new_pos = search (editor, gtk_text_get_point (GTK_TEXT (editor)), string, casein);
  if (new_pos > -1) {
    gtk_editable_set_position (GTK_EDITABLE (editor), new_pos);
    gtk_editable_select_region (GTK_EDITABLE (editor), new_pos,
				new_pos + strlen (string));
    return TRUE;
  } else {
    return FALSE;
  }
}

/* search_back -- searches in editor, from pos and backward, for
 * string.  If string is found, returns position, otherwise returns
 * -1.  If casein is true, search is case insensitive. */
/* FIXME: This could and should be optimised. There is no need to
 * get_text all the time, is there?  We just have to be carefull with
 * the gap. */
static gint
search_back (GtkEditor *editor, guint pos, gchar *string, gboolean casein)
{
  gchar *buf = NULL;
  gint match = -1;
  gint text_len = gtk_text_get_length (GTK_TEXT (editor));
  gint len = strlen (string);

  if (pos == 0) return -1; /* not illegal, but nothing to do */
  if (pos > text_len - len) pos = text_len - len; /* we must be reasonable */

  gtk_text_freeze (GTK_TEXT (editor));
  for (; pos >= 0; pos--) {
    buf = gtk_editable_get_chars (GTK_EDITABLE (editor), pos, pos + len);
    if (!buf) return match;	/* somethings wrong */
    if (casein) {
      if (strcasecmp (buf, string) == 0) {
	match = pos;
	break;
      }
    } else {
      if (strcmp (buf, string) == 0) {
	match = pos;
	break;
      }
    }
    g_free (buf);
  }
  g_free (buf);
  gtk_text_thaw (GTK_TEXT (editor));

  return match;
}

/* gtk_editor_search_back_from_point -- searches backward for string,
 * if found, goto and select match. If casein is true, search is case
 * insensitive. If string is found it returns TRUE, otherwise it
 * returns FALSE. */
gboolean
gtk_editor_search_back_from_point (GtkEditor *editor, 
				   gchar *string,
				   gboolean casein)
{
  gint new_pos;

  g_return_val_if_fail (editor != NULL, FALSE);
  g_return_val_if_fail (string != NULL, FALSE);

  new_pos = search_back (editor, gtk_text_get_point (GTK_TEXT (editor)),
			 string, casein);
  if (new_pos > -1) {
    gtk_editable_set_position (GTK_EDITABLE (editor), new_pos);
    gtk_editable_select_region (GTK_EDITABLE (editor), new_pos,
				new_pos + strlen (string));
    return TRUE;
  } else {
    return FALSE;
  }
}

/* gtk_editor_search_regex_from_point -- searches for string matching
 * regex, if found, goto and select match.  If the regex doesn't
 * compile (or a serious error occurs) it returns -1, if a match is
 * found it returns 1, and if no match is found it returns 0. */
gint
gtk_editor_regex_search_from_point (GtkEditor     *editor,
				    gchar         *regex)
{
  return _gtk_editor_select_regex (editor, regex, TRUE);
}

/* gtk_editor_search_regex_back_from_point -- searches for string
 * matching regex, if found, goto and select match.  If the regex
 * doesn't compile (or a serious error occurs) it returns -1, if a
 * match is found it returns 1, and if no match is found it returns
 * 0. */
gint
gtk_editor_regex_search_back_from_point (GtkEditor     *editor,
					 gchar         *regex)
{
  return _gtk_editor_select_regex (editor, regex, FALSE);
}


/* --<hilite stuff>------------------------------------------------------- */
GdkFont*
_gtk_editor_fontdup (GdkFont *font)
{
  if (font) {
    gdk_font_ref (font);
  }
  return font;
}

GdkColor*
_gtk_editor_coldup (const GdkColor *color)
{
  GdkColor *new;
  if (color) {
    new = g_new (GdkColor, 1);
    memcpy (new, color, sizeof (GdkColor));
  } else {
    new = NULL;
  }
  return new;
}


/* gtk_editor_install_stable -- installs the syntax table consisting
 * of the lists entries in editor. */
void
gtk_editor_install_stable (GtkEditor *editor,
			   GList *entries)
{
  _gtk_editor_destroy_stable (editor->stable);
  editor->stable = _gtk_editor_new_stable (entries);
}

/* gtk_editor_install_patterns -- installs highlight patterns for the
 * editor */
void
gtk_editor_install_patterns (GtkEditor *editor,
			     GList *entries)
{
  _gtk_editor_destroy_patterns (editor->patterns);
  editor->patterns = _gtk_editor_new_patterns (entries);
}

/* hilite_interval -- yep! That's what it does. FIXME: add description
 * of *how* it does it. */
void
gtk_editor_hilite_interval (GtkEditor *editor, guint from, guint to)
{
  guint pos;

  g_return_if_fail (editor != NULL);
  g_return_if_fail (from <= to);
  g_return_if_fail (to <= gtk_text_get_length (GTK_TEXT (editor)));

  pos = gtk_editable_get_position (GTK_EDITABLE (editor));

  gtk_text_freeze (GTK_TEXT (editor));

  /* correct start position, so we start at the beginning of a text
   * property If we are on offset 1, then the state should really be
   * the previous state.  It's a wonder to me, why it shouldn't be
   * offset 0...but this works, it doesn't with offset 0.  It's
   * probably because this is to fix deleting the last char of a
   * state, and this can only be done with backsspace, which puts us
   * in offset 1, or something. */
  gtk_text_set_point (GTK_TEXT (editor), from);
  if (GTK_TEXT (editor)->point.offset == 1) {
    GList *prev = GTK_TEXT (editor)->point.property->prev;
    if (prev) {
      /* it's a little hacky to get the length of the previous prop. */
      gint len = gtk_text_get_property_length (GTK_TEXT (editor),
					       from - 2);
      /* goto previous frame...the 1 is for the offset==1 thingie */
      from -= len + 1;
      if ((gint)from < 0) from = 0;	/* not before beginning of text */
    } else {
      from = 0;
    }
  } else {
    from -= GTK_TEXT (editor)->point.offset; /* go to beginning of property. */
  }

  /* syntax table highlighting */
  _gtk_editor_sthilite_interval (editor, from, to);

  gtk_editable_set_position (GTK_EDITABLE (editor), pos);
  gtk_text_thaw (GTK_TEXT (editor));
}

void
gtk_editor_hilite_buffer (GtkEditor *editor)
{
  gtk_editor_hilite_interval (editor, 0,
			      GTK_TEXT (editor)->text_end
			      - GTK_TEXT (editor)->gap_size);
  /* and just a little extra service */
  editor->changed_from = editor->changed_to = GTK_TEXT (editor)->point.index;
}

#define HALF_SCREEN_SIZE 100	/* ??? */

/* hiliting the entire buffer can be *very* slow...this guy can be used in stead.
 * Since I have no way of actually calculating the visible text, I make a guess...
 * FIXME: when the text API changes, this should be changed!
 */
void
gtk_editor_hilite_screen (GtkEditor *editor)
{
  gint idx = GTK_TEXT (editor)->point.index;
  
  gtk_editor_hilite_interval (editor, MAX (0, idx - HALF_SCREEN_SIZE),
			      MIN (idx + HALF_SCREEN_SIZE,
				   GTK_TEXT (editor)->text_end
				   - GTK_TEXT (editor)->gap_size));
  /* and just a little extra service */
  editor->changed_from = editor->changed_to = idx;
}


static gboolean
hilite_when_idle (GtkEditor *editor)
{
#ifdef EDITOR_DEBUG
  g_print ("hilite_when_idle [%u,%u]\n", 
	   editor->changed_from, editor->changed_to);
#endif

  if (editor->changed_from < editor->changed_to) {
    gtk_editor_hilite_interval (editor, editor->changed_from, editor->changed_to);
    editor->changed_from = editor->changed_to = 
      gtk_editable_get_position (GTK_EDITABLE (editor));
    editor->hilite_time = 0;
  }

  return FALSE;
}

/* --<indent stuff>------------------------------------------------------- */
/* FIXME: we should have some better api */

/* FIXME: crashes if called on the last line, if the begining of that
 * line contains a '\t'...maybe at gtktext bug... */
static void
default_one_line_ind (GtkEditor *editor)
{
  guint pos = gtk_editable_get_position (GTK_EDITABLE (editor));

  gtk_text_freeze (GTK_TEXT (editor));
  /* FIXME: change this when they change the interface to *_move_* */
  gtk_signal_emit_by_name (GTK_OBJECT (editor), "move_to_column", 0);
  gtk_text_set_point (GTK_TEXT (editor),
		      gtk_editable_get_position (GTK_EDITABLE (editor)));
  gtk_text_insert (GTK_TEXT (editor), NULL, NULL, NULL, "\t", 1);
  gtk_editable_set_position (GTK_EDITABLE (editor), pos + 1); /* old pos + tab */
  gtk_text_thaw (GTK_TEXT (editor));
}

/* indent-region -- calls the one line indenter repeatedly over the
 * selection.  This function should *only* be called if there *is* a
 * oneline indenter installed! */
static void
indent_region (GtkEditor *editor)
{
  guint from = MIN (GTK_EDITABLE (editor)->selection_start_pos,
		    GTK_EDITABLE (editor)->selection_end_pos);
  guint to = MAX (GTK_EDITABLE (editor)->selection_start_pos,
		  GTK_EDITABLE (editor)->selection_end_pos);
  guint old_pos = gtk_editable_get_position (GTK_EDITABLE (editor));
  guint pos = from;

  gtk_text_freeze (GTK_TEXT (editor));
  gtk_editable_set_position (GTK_EDITABLE (editor), pos);
  while (pos < to + 1) {	/* I *mean* +1 */
    editor->one_line_ind (editor);
    to += 1;			/* the tab remember! */

    /* FIXME: change this when they change the interface to *_move_* */
    gtk_signal_emit_by_name (GTK_OBJECT (editor), "move_cursor", 0, 1);
    pos = gtk_editable_get_position (GTK_EDITABLE (editor));
    gtk_text_set_point (GTK_TEXT (editor), pos);
  }
  gtk_editable_set_position (GTK_EDITABLE (editor), old_pos);
  gtk_text_thaw (GTK_TEXT (editor));
}

/* --<event handles>------------------------------------------------------ */
static gint
gtk_editor_key_press (GtkWidget *widget, GdkEventKey *event)
{
  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (event != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_EDITOR (widget), FALSE);

  if (event->keyval == GDK_Tab) {
    if (GTK_EDITOR (widget)->one_line_ind) {
      gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");

      if (GTK_EDITABLE (widget)->has_selection) {
	indent_region (GTK_EDITOR (widget));
      } else {
	GTK_EDITOR (widget)->one_line_ind (GTK_EDITOR (widget));
      }
      return TRUE;
    }
  }

  return FALSE;
}

/* To keep track of changes for hilite_when_idle */
static void
gtk_editor_changed_pre (GtkEditor *editor)
{
  if (editor->hilite_when_idle) {
    /* the first MIN looks silly, but think about deleting the last
     * char in a text, and you will understand. */
    editor->changed_from = MIN (MIN (editor->changed_from,
				     GTK_TEXT (editor)->point.index),
				gtk_text_get_length (GTK_TEXT (editor)) - 1);
  }
}

/* Keeps track of changes for hilite_when_idle, and installs hilite
 * function with a time delay. */
static void
gtk_editor_changed_post (GtkEditor *editor)
{
  if (editor->hilite_when_idle) {
    editor->changed_to = MIN (MAX (editor->changed_to,
				   GTK_TEXT (editor)->point.index),
			      gtk_text_get_length (GTK_TEXT (editor)));

    /* something changed...re-highlight when idle */
    if (editor->hilite_time) gtk_timeout_remove (editor->hilite_time);
    editor->hilite_time = gtk_timeout_add (TIME_DELAY, (GtkFunction) hilite_when_idle,
					   (gpointer) editor);
  }
}

==> gtkeditor-0.1.8.tar.gz: gtkeditor-0.1.8/gtkeditor.c <==
/* GtkEditor - a source editor widget for GTK
 * Copyright (C) 1998, 2000 Thomas Mailund.
 *
 * The editor widget was written by Thomas Mailund, so bugs should be
 * reported to <mailund@daimi.au.dk>, not the gtk ppl.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>

#include <glib.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>

#include "gtkeditor.h"
#include "internal.h"
#include "tktext-port/gtktextbtree.h"


/* widget stuff */
static void   gtk_editor_class_init          (GtkEditorClass   *klass);
static void   gtk_editor_init                (GtkEditor        *editor);
static void   gtk_editor_destroy             (GtkObject        *object);


/* --<local data>--------------------------------------------------------- */
static GtkWidgetClass *parent_class = NULL;

/* --<widget initialization, constructor and destructor>------------------ */
guint
gtk_editor_get_type ()
{
  static guint editor_type = 0;

  if (!editor_type)
    {
      GtkTypeInfo editor_info =
      {
	"GtkEditor",
	sizeof (GtkEditor),
	sizeof (GtkEditorClass),
	(GtkClassInitFunc) gtk_editor_class_init,
	(GtkObjectInitFunc) gtk_editor_init,
	(GtkArgSetFunc) NULL,
        (GtkArgGetFunc) NULL,
      };

      editor_type = gtk_type_unique (gtk_text_buffer_get_type (), &editor_info);
    }

  return editor_type;
}

static void
gtk_editor_class_init (GtkEditorClass *class)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*) class;

  /* init local data */
  parent_class = gtk_type_class (gtk_text_buffer_get_type ());


  /* setup signals */
  object_class->destroy = gtk_editor_destroy;
}

static void
gtk_editor_init (GtkEditor *editor)
{
  /* hilite */
  editor->scanner = NULL;
  editor->n_tag_entries = 0;
  editor->tag_entries = NULL;
  editor->tokens = NULL;
  editor->cur_token = NULL;
  editor->cur_token_pos = 0;
  editor->block_stack = NULL;

  gtk_signal_connect_after (GTK_OBJECT(editor),
                            "insert_text",
                            GTK_SIGNAL_FUNC(gtk_editor_insert_text),NULL);
  gtk_signal_connect (GTK_OBJECT(editor),
                            "delete_text",
                            GTK_SIGNAL_FUNC(gtk_editor_delete_text_length),NULL);
  gtk_signal_connect_after (GTK_OBJECT(editor),
                            "delete_text",
                            GTK_SIGNAL_FUNC(gtk_editor_delete_text),NULL);
}

GtkEditor*
gtk_editor_new ()
{
  GtkEditor *editor;

  editor = GTK_EDITOR (gtk_type_new (gtk_editor_get_type ()));
  editor->buffer.tag_table = gtk_text_tag_table_new();
  editor->buffer.tree = gtk_text_btree_new(editor->buffer.tag_table,
                                           &editor->buffer);
  editor->default_tag = gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(editor), "default");

  return editor;
}

static void
gtk_editor_destroy (GtkObject *object)
{
  g_return_if_fail (object != NULL);
  g_return_if_fail (GTK_IS_EDITOR (object));

  GTK_OBJECT_CLASS(parent_class)->destroy (object);
}
